/*
 * Copyright 2000-2006 Omnicore Software, Hans Kratz & Dennis Strein GbR,
 *                     Geert Bevin <gbevin[remove] at uwyn dot com>,
 *                                         Ulf Dittmer (ulf[remove] at ulfdittmer dot com).
 * Distributed under the terms of either:
 * - the common development and distribution license (CDDL), v1.0; or
 * - the GNU Lesser General Public License, v2.1 or later
 * $Id$
 */
package com.uwyn.jhighlight.highlighter;

import java.io.Reader;
import java.io.IOException;

%%

%class JavaScriptHighlighter
%extends SourceCodeHighlighter

%unicode
%pack

%buffer 128

%public

%int

%{
        /* Highlighter implementation */

        public byte getStartState()
        {
                return YYINITIAL+1;
        }

        public byte getCurrentState()
        {
                return (byte) (yystate()+1);
        }

        public void setState(byte newState)
        {
                yybegin(newState-1);
        }

        public byte getNextToken()
        throws IOException
        {
                return (byte) yylex();
        }

        public int getTokenLength()
        {
                return yylength();
        }

        public void setReader(Reader r)
        {
                this.zzReader = r;
        }

        public JavaScriptHighlighter()
        {
        }
%}

/* main character classes */

WhiteSpace = [ \t\f]

/* identifiers */

ConstantIdentifier = {SimpleConstantIdentifier}
SimpleConstantIdentifier = [A-Z0-9_]+

Identifier = [:jletter:][:jletterdigit:]*

/* int literals */

DecLiteral = 0 | [1-9][0-9]*

/* float literals */

FloatLiteral  = ({FLit1}|{FLit2}|{FLit3}|{FLit4})

FLit1 = [0-9]+ \. [0-9]* {Exponent}?
FLit2 = \. [0-9]+ {Exponent}?
FLit3 = [0-9]+ {Exponent}
FLit4 = [0-9]+ {Exponent}?

Exponent = [eE] [+\-]? [0-9]+

%state IN_COMMENT

%%

<YYINITIAL> {

  /* keywords */
  "break"                        |
  "case"                         |
  "catch"                        |
  "continue"                     |
  "do"                           |
  "else"                         |
  "finally"                      |
  "for"                          |
  "default"                      |
  "delete"                       |
  "new"                          |
  "goto"                         |
  "if"                           |
  "switch"                       |
  "return"                       |
  "while"                        |
  "this"                         |
  "try"                          |
  "var"                          |
  "function"                     |
  "with"                         |
  "in"                           |
  "true"                         |
  "false"                        |
  "null"                         { return KEYWORD_STYLE; }
  
  /* Built-in Types*/
  "Array"                        |
  "Boolean"                      |
  "RegExp"                       |
  "String"                       |
  {Identifier} ":"               { return TYPE_STYLE; }

  /* literals */
  (\" ( [^\"\n\\] | \\[^\n] )* (\n | \\\n | \")) |
  (\' ( [^\'\n\\] | \\[^\n] )* (\n | \\\n | \'))
                                 { return STRING_LITERAL_STYLE; }

  {DecLiteral} |
  {FloatLiteral}                 { return NUM_LITERAL_STYLE; }

  /* separators */
  "(" |
  ")" |
  "{" |
  "}" |
  "[" |
  "]" |
  ";" |
  "," |
  "."                          { return SEPARATOR_STYLE; }

  /* operators */
  "=" |
  ">" |
  "<" |
  "!" |
  "~" |
  "?" |
  ":" |
  "+" |
  "-" |
  "*" |
  "/" |
  "&" |
  "|" |
  "^" |
  "%"                      { return OPERATOR_STYLE; }

  {ConstantIdentifier}                    { return PLAIN_STYLE; }

  \n |
  {Identifier} |
  {WhiteSpace}                   { return PLAIN_STYLE; }

// single line comment

  "//" [^\n]* \n |

// short comment

  "/**/"        { return COMMENT_STYLE; }

// comment start

  "/*"      { yybegin(IN_COMMENT);  return COMMENT_STYLE;}

}


// normal comment mode

<IN_COMMENT> {

  // comment unterminated

  ([^\n*]|\*+[^\n*/])* (\n | \*+\n)  { return COMMENT_STYLE; }

  // comment terminated

  ([^\n*]|\*+[^\n*/])* \*+ "/"  { yybegin(YYINITIAL); return COMMENT_STYLE; }

}

/* error fallback */

.|\n                             { return PLAIN_STYLE; }

