package com.uwyn.jhighlight.highlighter;

import java.io.Reader;
import java.io.IOException;

%%

%class RubyHighlighter
%extends SourceCodeHighlighter

%unicode
%pack

%buffer 128

%public

%int

%{
        /* Highlighter implementation */

        public byte getStartState()
        {
                return YYINITIAL+1;
        }

        public byte getCurrentState()
        {
                return (byte) (yystate()+1);
        }

        public void setState(byte newState)
        {
                yybegin(newState-1);
        }

        public byte getNextToken()
        throws IOException
        {
                return (byte) yylex();
        }

        public int getTokenLength()
        {
                return yylength();
        }

        public void setReader(Reader r)
        {
                this.zzReader = r;
        }

        public RubyHighlighter()
        {
        }
%}

/* main character classes */
LineTerminator = \r|\n|\r\n
InputCharacter = [^\r\n]

WhiteSpace = {LineTerminator} | [ \t\f]+

/* comments */
Comment = "#" {InputCharacter}* {LineTerminator}?

/* identifiers */
Identifier = [a-zA-Z][a-zA-Z0-9_]*

DecLiteral = 0 | [1-9][0-9]*
FloatLiteral  = ({FLit1}|{FLit2}|{FLit3}|{FLit4})

FLit1 = [0-9]+ \. [0-9]* {Exponent}?
FLit2 = \. [0-9]+ {Exponent}?
FLit3 = [0-9]+ {Exponent}
FLit4 = [0-9]+ {Exponent}?

Exponent = [eE] [+\-]? [0-9]+


%%

<YYINITIAL> {

  /* keywords */
  "BEGIN"                        |
  "ensure"                       |
  "assert"                       |
  "nil"                          |
  "self"                         |
  "when"                         |
  "END"                          |
  "false"                        |
  "not"                          |
  "super"                        |
  "alias"                        |
  "defined"                      |
  "or"                           |
  "then"                         |
  "yield"                        |
  "and"                          |
  "redo"                         |
  "true"                         |
  "else"                         |
  "in"                           |
  "rescue"                       |
  "undef"                        |
  "break"                        |
  "elsif"                        |
  "module"                       |
  "retry"                        |
  "unless"                       |
  "next"                         |
  "return"                       { return KEYWORD_STYLE; }

  "begin"                        |
  "case"                         |
  "class"                        |
  "def"                          |
  "for"                          |
  "while"                        |
  "until"                        |
  "do"                           |
  "if"                           { return KEYWORD_STYLE; }

  "end"                          { return KEYWORD_STYLE; }


  /* Built-in Types*/
  "self"                         |
  "nil"                          |
  "true"                         |
  "false"                        |
  "__FILE__"                     |
  "__LINE__"                     {  return TYPE_STYLE;  }



  /* operators */

  "("                            { return OPERATOR_STYLE; }
  ")"                            { return OPERATOR_STYLE; }
  "{"                            { return OPERATOR_STYLE; }
  "}"                            { return OPERATOR_STYLE; }
  "["                            { return OPERATOR_STYLE; }
  "]"                            { return OPERATOR_STYLE; }
  "+"                            |
  "-"                            |
  "*"                            |
  "**"                           |
  "/"                            |
  "//"                           |
  "%"                            |
  "<<"                           |
  ">>"                           |
  "&"                            |
  "|"                            |
  "^"                            |
  "~"                            |
  "<"                            |
  ">"                            |
  "<="                           |
  ">="                           |
  "=="                           |
  "!="                           |
  "<>"                           |
  "@"                            |
  ","                            |
  ":"                            |
  "."                            |
  ".."                           |
  "`"                            |
  "="                            |
  ";"                            |
  "+="                           |
  "-="                           |
  "*="                           |
  "/="                           |
  "//="                          |
  "%="                           |
  "&="                           |
  "|="                           |
  "^="                           |
  ">>="                          |
  "<<="                          |
  "**="                          { return OPERATOR_STYLE; }

  /* string literal */
    /* literals */
  (\" ( [^\"\n\\] | \\[^\n] )* (\n | \\\n | \")) |
  (\' ( [^\'\n\\] | \\[^\n] )* (\n | \\\n | \'))
                                 { return STRING_LITERAL_STYLE; }

  {DecLiteral} |
  {FloatLiteral}                 { return NUM_LITERAL_STYLE; }

  /* comments */
  {Comment}                      { return COMMENT_STYLE; }

  /* whitespace */
  {WhiteSpace}                   { return PLAIN_STYLE; }

  /* identifiers */
  {Identifier}"?"                { return PLAIN_STYLE; }
  {Identifier}                   { return PLAIN_STYLE; }
}

/* error fallback */

.|\n                             { return PLAIN_STYLE; }

